Module future

Module future 

Traits

AssertSend
If your future is not Send enough, try this.
AsyncFriendly
The bounds Send + Sync + 'static are often required by type parameters to make Futures Send + Sync + 'static.
SendFuture
A simplified type alias for Future<Output = T> + Send.

Functions

boxit
Type erase the provided future by boxing it.