PooledArc

Struct PooledArc 

pub struct PooledArc<T> { /* private fields */ }
PooledArc<T> is a pooled object wrapper with an Arc to its parent pool.

When dropped, the contained object will be returned to the pool if there is space.

Unlike PooledRef, this object is 'static (it T: 'static).