Undef

Struct Undef 

pub struct Undef {
    pub len: usize,
}
An AsPooled initializer for Vec<T>.

Creates or modifies an existing array to be the specified length, without specifying the contents of the returned array.

This lack of specification means that existing Vecs that are modified to be the configured length will contain values from a previous run.

Fields

len: usize

Implementations

Source

impl Undef

pub fn new(len: usize) -> Self

Construct a new [Defaulted].