Module reborrow

Module reborrow 

A collection of tools for working with generalized references and scrounging through types.

Structs

Cloned
A container that reborrows by cloning the contained value.
Copied
A container that reborrows by copying the contained value.
Place
A container for types T providing an implementation Deref<Target = T> as well as reborrowing functionality mapped to T as Deref and T as DerefMut.

Traits

Reborrow
An hybrid combination of reference covariance and borrowing for generalized reference types.
ReborrowMut
An hybrid combination of reference covariance and borrowing for generalized reference types.