AsyncLower

Trait AsyncLower 

pub trait AsyncLower<'short, T: ?Sized, Lifetime: Sealed = BoundTo<&'short Self>> {
    type Proxy: Deref<Target = T> + Send + Sync;

    // Required method
    fn async_lower(&'short self) -> Self::Proxy;
}
An async-friendly version of Lower that further constrains the proxy to be Send and Sync.

This trait is automatically implemented for types implementing Lower whose proxies are already Send and Sync.

Notes

As of time of writing, adding constraints like

T: Lower<'a, T, Proxy: Send + Sync>

is not sufficient to attach Send and Sync bounds to the proxy.

Required Associated Types

type Proxy: Deref<Target = T> + Send + Sync

Required Methods

fn async_lower(&'short self) -> Self::Proxy

Implementors

Source

impl<'short, T, U> AsyncLower<'short, T> for U
where T: ?Sized, U: Lower<'short, T, Proxy: Send + Sync>,

Source

type Proxy = <U as Lower<'short, T>>::Proxy