pub struct Layout<T> { /* private fields */ }Expand description
The layout for Strided.
This struct ensures that the Self::cstride is greater than or equal to Self::ncols
and that the addressable span is valid for elements of type T. In particular, the
linear length does not overflow usize::MAX and its size in bytes does not exceed
isize::MAX.
The linear length of Strided is given by the formula
self.nrows.saturating_sub(1) * self.cstride + self.nrows.min(1) * self.ncolsThis allows the last row to occupy less than a full stride.
Implementations
Sourceimpl<T> Layout<T>
impl<T> Layout<T>
Sourcepub fn new(
nrows: usize,
ncols: usize,
cstride: usize,
) -> Result<Self, LayoutError>
pub fn new( nrows: usize, ncols: usize, cstride: usize, ) -> Result<Self, LayoutError>
Construct a new Layout.
Errors if:
cstride < ncols.- The computation of the linear length overflows
usize::MAX. - The number of bytes required for the full span exceeds
isize::MAX.
Sourcepub fn linear_length(&self) -> usize
pub fn linear_length(&self) -> usize
Return the length of the addressable span described by this Layout, including
gaps between rows.