Strided

Struct Strided 

pub struct Strided<'a, T> { /* private fields */ }
A row-major strided matrix.

This is a generalization of the Matrix trait as it does not mandate a dense layout in memory.

           |<------ cstride ----->|
                    |<-- ncols -->|
                    +-------------+
slice 0 -> a0 a1 a2 | a3 a4 a5 a6 |    ^
slice 1 -> b0 b1 b2 | b3 b4 b4 b5 |    |
slice 2 -> c0 c1 c2 | c3 c4 c5 c6 |  nrows
slice 3 -> d0 d1 d2 | d3 d4 d5 d6 |    |
slice 4 -> e0 e1 e2 | e3 e4 e5 e6 |    |
slice 5 -> f0 f1 f2 | f3 f4 f5 f6 |    v
                    +-------------+
                          ^
                          |
                       Strided

This abstraction is useful when performing PQ related operations such as training or compression as it provides a convenient abstraction for working with columnar subsets of dense data in-place.

Implementations

Source

impl<'a, T> Strided<'a, T>

pub fn try_from_data( data: &'a [T], nrows: usize, ncols: usize, cstride: usize, ) -> Result<Self, TryFromError>

Construct a strided view over data slice, shrinking the slice as needed.

Returns an error if the layout is invalid or data is not at least Layout::linear_length.

pub fn layout(&self) -> Layout<T>

Return the Layout for the matrix.

pub fn as_ptr(&self) -> *const T

Return a pointer to the base of the matrix.

pub fn ncols(&self) -> usize

Return the number of columns in the matrix.

pub fn nrows(&self) -> usize

Return the number of rows in the matrix.

pub fn cstride(&self) -> usize

Return the count of elements between the start of each row.

pub fn as_slice(&self) -> &[T]

Return the underlying data as a slice.

Note

The underlying representation for a strided matrix is not necessarily dense.

pub unsafe fn element_unchecked(&self, row: usize, col: usize) -> &T

Return the specified element.

Safety
  • row < self.nrows().
  • col < self.ncols().
pub fn get_element(&self, row: usize, col: usize) -> Option<&T>

Return the specified element if row < self.nrows() and col < self.ncols().

pub fn element(&self, row: usize, col: usize) -> &T

Return the specified element.

Panics

Panics if row >= self.nrows() or col >= self.ncols().

pub unsafe fn row_unchecked(&self, row: usize) -> &[T]

Returns the requested row without boundschecking.

Safety

Caller must ensure row < self.nrows().

pub fn get_row(&self, row: usize) -> Option<&[T]>

Return the requested row if row < self.nrows().

pub fn row(&self, row: usize) -> &[T]

Return row row as a slice.

Panics

Panics if row >= self.nrows().

pub fn rows(&self) -> Rows<'_, T> ⓘ

Return a iterator over all rows in the matrix.

Rows are yielded sequentially beginning with row 0.