TryFromError

Struct TryFromError 

#[non_exhaustive]
pub struct TryFromError<T: DenseData> { /* private fields */ }

Implementations

Source

impl<T: DenseData> TryFromError<T>

pub fn into_inner(self) -> T

Consume the error and return the base data.

pub fn as_static(&self) -> TryFromErrorLight

Drop the data portion of the error and return an equivalent error that is guaranteed to be 'static.