pub struct LazyString<F>(/* private fields */)
where
F: Fn(&mut Formatter<'_>) -> Result<(), Error>;Expand description
A struct used to lazily defer creation of custom async logging messages until we know that the message is actually needed.
Context
Logging in the async context explicitly requires passing of a context pointer to enable CDB to determine the source of error message. To that end, a custom logging function is used.
The LazyString captures a lambda that constructs the logging message and implements
std::fmt::Display, allowing string formatting to only be performed once we know a
message needs to be logged.
Implementations