Skip to main contentModule views
Source - Init
- A matrix initializer that invokes the provided lambda to initialize each element.
- MatrixBase
- A view over dense chunk of memory, interpreting that memory as a 2-dimensional matrix
laid out in row-major order.
- TryFromError
- TryFromErrorLight
- DenseData
- Various view types (types such as
MatrixView that add semantic meaning to blobs
of data) need both immutable and mutable variants. - Generator
- A generator for initializing the entries in a matrix via
Matrix::new. - MutDenseData
- A mutable companion to
DenseData.
- Matrix
- Represents an owning, 2-dimensional view of a contiguous block of memory,
interpreted as a matrix in row-major order.
- MatrixView
- Represents a non-owning, 2-dimensional view of a contiguous block of memory,
interpreted as a matrix in row-major order.
- MutMatrixView
- Represents a mutable non-owning, 2-dimensional view of a contiguous block of memory,
interpreted as a matrix in row-major order.