pub struct RowCol {
pub row: usize,
pub col: usize,
}Expand description
An initializer argument for the closure provided to Owned::from_fn and
Owned::try_from_fn to remove ambiguity of the row and column being initialiazed.
Fields
row: usizecol: usizeTrait Implementations
impl Copy for RowCol
impl Eq for RowCol
impl StructuralPartialEq for RowCol