pub type Matrix<T> = MatrixBase<Box<[T]>>;Expand description
Represents an owning, 2-dimensional view of a contiguous block of memory, interpreted as a matrix in row-major order.
Aliased Type
pub struct Matrix<T> { /* private fields */ }pub type Matrix<T> = MatrixBase<Box<[T]>>;Represents an owning, 2-dimensional view of a contiguous block of memory, interpreted as a matrix in row-major order.
pub struct Matrix<T> { /* private fields */ }