Shared utilities for DiskANN crates.
Re-exports
pub use reborrow::Reborrow;pub use reborrow::ReborrowMut;
Modules
- future
- io
- Read and write vectors in the DiskANN binary format.
- object_
pool - The
ObjectPool<C>class is a thread-safe queue that allows concurrent access and is used for reusing allocated objects. - reborrow
- A collection of tools for working with generalized references and scrounging through types.
- sampling
- strided
- views
Macros
- lazy_
format - A macro that behaves like
format!but constructs aLazyStringto defer string formatting until the result is actually displayed. If theLazyStringis never displayed, this construct has minimal overhead.
Structs
- Lazy
String - A struct used to lazily defer string formatting until needed. This is used to implement
lazy_format!: a lazy version of the standardformat!macro.