Skip to main contentCrate diskann_utils
Source pub use reborrow::Reborrow;pub use reborrow::ReborrowMut;pub use lifetime::WithLifetime;
- future
- io
- Read and write vectors in the DiskANN binary format.
- lifetime
- object_pool
- The
ObjectPool<C> class is a thread-safe queue that allows concurrent access and is
used for reusing allocated objects. - reborrow
- A collection of tools for working with generalized references and scrounging through types.
- sampling
- strided
- views
- lazy_format
- A macro that behaves like
format! but constructs a diskann::utils::LazyString
to enable deferred evaluation of the error message.
- LazyString
- A struct used to lazily defer creation of custom async logging messages until we know
that the message is actually needed.