pub struct Undef {
pub len: usize,
}Expand description
An AsPooled initializer for Vec<T>.
Creates or modifies an existing array to be the specified length, without specifying the contents of the returned array.
This lack of specification means that existing Vecs that are modified to be the
configured length will contain values from a previous run.
Fields
len: usizeImplementations