pub trait AsyncLower<'short, T: ?Sized, Lifetime: Sealed = BoundTo<&'short Self>> {
type Proxy: Deref<Target = T> + Send + Sync;
// Required method
fn async_lower(&'short self) -> Self::Proxy;
}Expand description
An async-friendly version of Lower that further constrains the proxy to be Send and
Sync.
This trait is automatically implemented for types implementing Lower whose proxies
are already Send and Sync.
Notes
As of time of writing, adding constraints like
T: Lower<'a, T, Proxy: Send + Sync>is not sufficient to attach Send and Sync bounds to the proxy.