StridedBase

Struct StridedBase 

pub struct StridedBase<T>
where T: DenseData,
{ /* private fields */ }
A row-major strided matrix.

This is a generalization of the MatrixBase class as it does not mandate a dense layout in memory.

           |<------ cstride ----->|
           |<-- ncols -->|
           +-------------+
slice 0 -> | a0 a1 a2 a3 | a4 a5 a6     ^
slice 1 -> | b0 b1 b2 b3 | b4 b5 b6     |
slice 2 -> | c0 c1 c2 c3 | c4 c5 c6   nrows
slice 3 -> | d0 d1 d2 d3 | d4 d5 d6     |
slice 4 -> | e0 e1 e2 e3 | e4 e5 e6     |
slicf 5 -> | f0 f1 f2 f3 | f4 f5 f6     v
           +-------------+
                 ^
                 |
            StridedView

This abstraction is useful when performing PQ related operations such as training or compression as it provides a convenient abstraction for working with columnar subsets of dense data in-place.

Implementations

Source

impl<'a, T> StridedBase<&'a [T]>

pub fn try_shrink_from( data: &'a [T], nrows: usize, ncols: usize, cstride: usize, ) -> Result<Self, TryFromError<&'a [T]>>

Construct a strided view over data slice, shrinking the slice as needed.

Returns an error if data is shorter than the value returned by linear_length.

Panics
  • Panics if cstride < ncols.
Source

impl<'a, T> StridedBase<&'a mut [T]>

pub fn try_shrink_from_mut( data: &'a mut [T], nrows: usize, ncols: usize, cstride: usize, ) -> Result<Self, TryFromError<&'a mut [T]>>

Construct a strided view over data slice, shrinking the slice as needed.

Returns an error if data is shorter than the value returned by linear_length.

Panics
  • Panics if cstride < ncols.
Source

impl<T> StridedBase<T>
where T: DenseData,

pub fn try_from( data: T, nrows: usize, ncols: usize, cstride: usize, ) -> Result<Self, TryFromError<T>>

Construct a strided view over data slice, shrinking the slice as needed.

Returns an error if data is not equal to the expected length as determined by linear_length.

Panics
  • Panics if cstride < ncols.
pub fn ncols(&self) -> usize

Return the number of columns in the matrix.

pub fn nrows(&self) -> usize

Return the number of rows in the matrix.

pub fn cstride(&self) -> usize

Return the count of elements between the start of each row.

pub fn as_slice(&self) -> &[T::Elem]

Return the underlying data as a slice.

Note

The underlying representation for a strided matrix is not necessarily dense.

pub fn as_mut_slice(&mut self) -> &mut [T::Elem]
where T: MutDenseData,

Return the underlying data as a slice.

Note

The underlying representation for a strided matrix is not necessarily dense.

pub fn row(&self, row: usize) -> &[T::Elem]

Return row row as a slice.

Panic

Panics if row >= self.nrows().

pub unsafe fn get_row_unchecked(&self, row: usize) -> &[T::Elem]

Returns the requested row without boundschecking.

Safety

The following conditions must hold to avoid undefined behavior:

  • row < self.nrows().
pub fn row_mut(&mut self, row: usize) -> &mut [T::Elem]
where T: MutDenseData,

Return row row as a mutable slice.

Panics

Panics if row >= self.nrows().

pub unsafe fn get_row_unchecked_mut(&mut self, row: usize) -> &mut [T::Elem]
where T: MutDenseData,

Returns the requested row without boundschecking.

Safety

The following conditions must hold to avoid undefined behavior:

  • row < self.nrows().
pub fn row_iter(&self) -> impl Iterator<Item = &[T::Elem]>

Return a iterator over all rows in the matrix.

Rows are yielded sequentially beginning with row 0.

Panics

Panics if self.ncols() == 0 (because the implementation does not work correctly in this case and it’s too corner-case to bother fixing). This restriction may be lifted in the future.

pub fn row_iter_mut(&mut self) -> impl Iterator<Item = &mut [T::Elem]>
where T: MutDenseData,

Return a mutable iterator over all rows in the matrix.

Rows are yielded sequentially beginning with row 0.

Panics

Panics if self.ncols() == 0 (because the implementation does not work correctly in this case and it’s too corner-case to bother fixing). This restriction may be lifted in the future.

pub fn as_ptr(&self) -> *const T::Elem

Return a pointer to the base of the matrix.

pub fn as_mut_ptr(&mut self) -> *mut T::Elem
where T: MutDenseData,

Return a pointer to the base of the matrix.

pub unsafe fn get_unchecked(&self, row: usize, col: usize) -> &T::Elem

Returns a reference to an element without boundschecking.

Safety

The following conditions must hold to avoid undefined behavior:

  • row < self.nrows().
  • col < self.ncols().
pub unsafe fn get_unchecked_mut( &mut self, row: usize, col: usize, ) -> &mut T::Elem
where T: MutDenseData,

Returns a mutable reference to an element without boundschecking.

Safety

The following conditions must hold to avoid undefined behavior:

  • row < self.nrows().
  • col < self.ncols().
pub fn as_view(&self) -> StridedView<'_, T::Elem>

Return a view over the matrix.