Module views
diskann_
utils
0.60.0
Module views
Module Items
Modules
Traits
In crate diskann_
utils
diskann_utils
Module
views
Modules

rowmajor
Traits

Dense
Data
This trait can be implemented by wrappers for immutable and mutable slice references, allowing for a common code path for immutable and mutable view types.
MutDense
Data
A mutable companion to
DenseData
.