pub type MutMatrixView<'a, T> = MatrixBase<&'a mut [T]>;Expand description
Represents a mutable non-owning, 2-dimensional view of a contiguous block of memory, interpreted as a matrix in row-major order.
This type is useful for functions that need to modify matrix data without taking ownership.
By accepting a MutMatrixView, such functions can operate on both owned matrices (by converting them
to a MutMatrixView) and existing non-owning mutable views.
Aliased Type
pub struct MutMatrixView<'a, T> { /* private fields */ }